// ----------------------------------------------------------------------
// StartUp! - Application object
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// Handle application duties for StartUp!; keeps track of our StartupItems,
// resources, etc.
// ----------------------------------------------------------------------

#ifndef StartUp_App_H
#define StartUp_App_H

#include <app/Application.h>
#include <app/Message.h>
#include <storage/FindDirectory.h>
#include <support/String.h>
#include <support/SupportDefs.h>

class BAppFileInfo;
class BFile;
class BPath;
class MutexList;
class StartupSettings;

// Some BMessages that our app understands.
enum {
	App_SHOW_HOMEPAGE	= 'suHP',
	App_SHOW_HELP		= 'shH?',
};

class App : public BApplication
{
public:
	App( void );
	virtual ~App( void );

	// BApplication hooks
	virtual void AboutRequested( void );
	virtual void ArgvReceived( int32 argc, char **argv );
	virtual void MessageReceived( BMessage *msg );
	virtual bool QuitRequested( void );
	virtual void ReadyToRun( void );
	virtual void RefsReceived( BMessage *msg );

	// StartUp application methods
	int ExitValue( void );				// value to return to the shell
	void SetExitValue( int val );		// set the exit value
	BAppFileInfo *GetInfo( void );		// lazy-allocate _info
	BFile *GetAppFile( void );			// lazy-allocate _app_file
	BPath *GetStartupPath( void );		// lazy-allocate _startup_path
	MutexList *GetStartupItems( void );	// lazy-allocate _startup_items

	bool ShowDocs( void );			// display documentation
	bool ShowADSWebsite( void );	// display the Arcane Dragon Software website

	bool CheckStartupPath( void );	// check the path for startup items

	// Convert a UserBootscript into multiple Startup Items
	status_t ConvertScript( directory_which where = B_USER_BOOT_DIRECTORY, 
							const char *name = "UserBootscript" );
	bool CreateItem( const char *name, const char *src, BEntry *new_ent = NULL );
	status_t NewItem( BMessage& command );

	// Keep track of whether we have any windows open or not.
	void RegisterWindow( BWindow *win );
	void DeregisterWindow( BWindow *win );

	// Settings methods
	StartupSettings *GetSettings( void );

	// Registration Key
	bool CheckKey( void );
	bool TestKey( BString& name, BString& company, BString &copies, BString &key );
	void SetIsNagging( bool val );

	// Install yourself
	void DoInstall( void );

private:
	App( const App& );          // don't copy me
	App& operator=( App& );     // don't assign me

	int _exit_value;			// value returned to the shell when we exit

	BAppFileInfo *_info;		// icon, version, etc. for the application
	BFile *_app_file;			// application file; needed for _info
	BPath *_startup_path;		// StartUp items directory
	MutexList *_startup_items;	// StartUp items list

	size_t _app_sig_size;		// Used for loading the signature from resources.

	bool _do_launch;			// should we do the launch window?
	bool _do_install;			// should we do the install window?
	bool _is_nagging;			// are we still nagging the user?

	MutexList *_windows;		// list of windows we have

	StartupSettings *_settings;	// application settings
};

#endif
