// ----------------------------------------------------------------------
// AppIconView.h - application icon view for StartUp!
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// The application's icon.
// ----------------------------------------------------------------------

#include "AppIconView.h"
#include "App.h"

#include <interface/Bitmap.h>
#include <interface/InterfaceDefs.h>
#include <interface/Screen.h>
#include <interface/Window.h>
#include <storage/AppFileInfo.h>
#include <string.h>

// ----------------------------------------------------------------------
// Static methods

BBitmap *AppIconView::GetIcon( icon_size which )
{
	BBitmap *icon = new BBitmap( BRect( 0, 0, which - 1, which - 1 ), B_CMAP8 );

	bool got_icon = false;

	BAppFileInfo *info = static_cast<App *>( be_app )->GetInfo();
	if( info != NULL ) {
		status_t retval = info->GetIcon( icon, which );

		if( retval == B_OK ) got_icon = true;
	}

	if( !got_icon ) {
		// Make up an icon...
		memset( icon->Bits(), 
				BScreen().IndexForColor( ui_color( B_WINDOW_TAB_COLOR ) ), 
				icon->BitsLength() );
	}

	return icon;
}

// ----------------------------------------------------------------------
// Constructor/destructor

AppIconView::AppIconView( icon_size which )
	: BView( BRect( 0, 0, which - 1, which - 1 ), "icon view",
			 B_FOLLOW_ALL_SIDES,
			 B_WILL_DRAW ),
	  _icon_bits( NULL ),
	  _which( which )
{
	// Get the application's icon
	_icon_bits = GetIcon( which );

	SetViewColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetHighColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetLowColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetDrawingMode( B_OP_OVER );
}

AppIconView::~AppIconView( void )
{
	if( _icon_bits ) delete _icon_bits;
}

// ----------------------------------------------------------------------
// BView methods

void AppIconView::Draw( BRect update )
{
	if( Window()->LockLooper() ) {
		DrawBitmapAsync( _icon_bits, update, update );
		Window()->UnlockLooper();
	}
}

void AppIconView::GetPreferredSize( float *width, float *height )
{
	*width = _icon_bits->Bounds().Width() + 1.0;
	*height = _icon_bits->Bounds().Height() + 1.0;
}

void AppIconView::ResizeToPreferred( void )
{
	// Do nothing; we don't change our size.
	float width, height;
	GetPreferredSize( &width, &height );
	ResizeTo( width, height );

	return;
}

void AppIconView::MouseDown( BPoint where )
{
	// Tell our window about it
	BMessage msg( *(Window()->CurrentMessage()) );
	msg.what = APP_ICON_CLICK;
	Window()->PostMessage( &msg );

	return;
}
