// ----------------------------------------------------------------------
// CommandParser - command-line parser
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// Simple Bourne-shell command-line parser; ignores complex structures
// (such as for, while, case, if), parses out simple command/arg lines.
//
// This isn't really intended as a generic /bin/sh parser, it's aimed at
// building data for StartUp!.
// ----------------------------------------------------------------------

#ifndef CommandParser_H
#define CommandParser_H

#include <app/Message.h>
#include <stack>

class BString;

class CommandParser
{
public:
	CommandParser( void );
	virtual ~CommandParser( void );

	// Returns the "cmd" and "args" in out, "directory" if a "cd" or "pushd"
	// occurred before.  Return values:
	//
	// B_OK			- finished parsing this statement
	// B_ERROR		- some error occured, out doesn't have useful data
	// B_BUSY		- line was continued
	// B_CANCELED	- line was a comment or blank
	//
	// If you get B_BUSY back, you should pass the same BMessage with the
	// next line(s) until you get something other than B_BUSY.
	//
	// out fields:
	//	"cmd"		B_STRING_TYPE	command name
	//	"args"		B_STRING_TYPE	individual arguments
	//	"directory"	B_STRING_TYPE	directory for command to run in
	//	"orig_args"	B_STRING_TYPE	original untouched command args(s),
	//								minus the command tokan
	//  "complex"	B_BOOL_TYPE		is the command "complex"?
	//	"&"			B_BOOL_TYPE		run in the background?
	//	"redirects"	B_STRING_TYPE	all > and < items
	virtual status_t Parse( const char *str, BMessage &out, bool need_cmd = true );

private:
	CommandParser( const CommandParser& );		// don't copy me
	CommandParser& operator=( CommandParser& );	// don't assign me

	enum cmd_parser_state {
		CMD_CLEAN,				// not "in" anything
		CMD_IN_SUBSHELL,		// in a () subshell
		CMD_IN_FUNCTION,		// in {} function definition
		CMD_IN_IF,				// in "if" statement
		CMD_IN_FOR,				// in "for" statement
		CMD_IN_WHILE,			// in "while" statement
		CMD_IN_CASE,			// in "case" statement
		CMD_CONTINUED,			// found \ at end of line
		CMD_IN_COMMENT,			// in a comment
		CMD_SKIP_REDIRECT,		// skip a > or < item
	};

	typedef stack<cmd_parser_state> state_stack;
	typedef stack<char> escape_stack;
	
	state_stack _state;
	BString *_last_dir;			// last "cd" or "pushd" command destination
};

#endif
