// ----------------------------------------------------------------------
// ItemIconView.h - Startup Item icon view for StartUp!
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// The item's icon.
// ----------------------------------------------------------------------

#include "ItemIconView.h"

#include <interface/Bitmap.h>
#include <interface/InterfaceDefs.h>
#include <interface/Screen.h>
#include <interface/Window.h>
#include <storage/Mime.h>
#include <string.h>

// ----------------------------------------------------------------------
// Constructor/destructor

ItemIconView::ItemIconView( void )
	: BView( BRect( 0.0, 0.0, B_LARGE_ICON - 1.0, B_LARGE_ICON - 1.0 ),
			 "icon view",
			 B_FOLLOW_ALL_SIDES,
			 B_WILL_DRAW ),
	  _icon_bits( NULL )
{
	SetViewColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetLowColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetHighColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );
	SetDrawingMode( B_OP_OVER );
}

ItemIconView::~ItemIconView( void )
{
}

// ----------------------------------------------------------------------
// BView methods

void ItemIconView::Draw( BRect update )
{
	if( !_icon_bits ) {
		FillRect( update );
	} else {
		if( Window()->LockLooper() ) {
			DrawBitmapAsync( _icon_bits, update, update );
			Window()->UnlockLooper();
		}
	}
}

void ItemIconView::GetPreferredSize( float *width, float *height )
{
	*width = B_LARGE_ICON;	// Is this off-by-one?
	*height = B_LARGE_ICON;	// Is this off-by-one?
}

void ItemIconView::ResizeToPreferred( void )
{
	// Do nothing; we don't change our size.
	float width, height;
	GetPreferredSize( &width, &height );
	ResizeTo( width, height );

	return;
}

// ----------------------------------------------------------------------
// ItemIconView methods

void ItemIconView::SetIcon( BBitmap *icon )
{
	FillRect( Bounds() );
	_icon_bits = icon;
	Draw( Bounds() );
}
