// ----------------------------------------------------------------------
// StartUp! - Launch window
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// Display and manage the Launch window.
// ----------------------------------------------------------------------

#ifndef StartUp_LaunchWindow_H
#define StartUp_LaunchWindow_H

#include "MWindow.h"

#include <kernel/OS.h>

class BFileGameSound;
class ItemIconView;
class MutexList;
class MProgressBar;
class MStringView;
class StartupItem;

class LaunchWindow : public MWindow
{
public:
	LaunchWindow( MutexList *items );
	virtual ~LaunchWindow( void );

	// BWindow methods
	virtual bool QuitRequested( void );

	// LaunchWindow methods
	bool IsQuitting( void ) { return _quitting; }	// so the thread can exit
	MutexList *Items( void ) { return _items; }		// so the thread can get the items
	void UpdateDisplay( StartupItem *item, int n, int m );	// update the display before we launch this item
	void Done( void );								// update the display because we're done

private:
	LaunchWindow( const LaunchWindow& );          // don't copy me
	LaunchWindow& operator=( LaunchWindow& );     // don't assign me

	size_t _size;					// Used for loading resources

	thread_id _tid;					// launch thread ID
	bool _quitting;					// are we quitting?

	MutexList *_items;				// Startup Items

	const char *_nofm_format;		// n of m view printf() format string
	
	MStringView *_action_view;
	ItemIconView *_icon_view;
	MStringView *_item_name_view;
	MStringView *_nofm_view;
	MProgressBar *_status_view;

	BFileGameSound *_sfx;			// for our launch SFX

	thread_id LaunchThread( void );	// launch our launcher thread
};

#endif
