// ----------------------------------------------------------------------
// MutexList.cpp
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// A BList that's guarded by its own mutex.  Not rocket science.
// ----------------------------------------------------------------------

#include "MutexList.h"

#include <support/Locker.h>

MutexList::MutexList( const char *name, int count )
	: BList( count ),
	  _mutex( NULL )
{
	_mutex = new BLocker( name, true );
}

MutexList::~MutexList( void )
{
	if( _mutex ) delete _mutex;
}
	
// ----------------------------------------------------------------------
// Mutex operations

bool MutexList::Lock( void )
{
	return _mutex->Lock();
}

status_t MutexList::LockWithTimeout( bigtime_t timeout )
{
	return _mutex->LockWithTimeout( timeout );
}

void MutexList::Unlock( void )
{
	_mutex->Unlock();
}

const BLocker *MutexList::Mutex( void )
{
	return _mutex;
}
