// ----------------------------------------------------------------------
// Startup! - Nag window
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// Display and manage the Nag window.
// ----------------------------------------------------------------------

#include "App.h"
#include "AppIconView.h"
#include "NagWindow.h"
#include "StartupSettings.h"

#include <app/Message.h>
#include <app/MessageRunner.h>
#include <app/Messenger.h>
#include <game/FileGameSound.h>
#include <interface/Font.h>
#include <interface/Screen.h>
#include <storage/Resources.h>
#include <support/Autolock.h>
#include <support/String.h>

#include "HGroup.h"
#include "MBViewWrapper.h"
#include "MBorder.h"
#include "MButton.h"
#include "MStringView.h"
#include "MTextView.h"
#include "Space.h"
#include "VGroup.h"

// +-----------------------------+
// | [ ]  StartUp! is ShareWare  |
// +-----------------------------+------------------+
// |                                                |
// |  +--+                                          |
// |  |1 |  StartUp! is ShareWare                   |
// |  +--+                                          |
// |        Arcane Dragon Software's StartUp! is    |
// |        shareware.  You get to try it first     |
// |        before paying for it, if you decide to  |
// |        keep it on your system.                 |
// |                                                |
// |        If you register StartUp!, you're        |
// |        supporting development of BeOS          |
// |        software, a very important thing for a  |
// |        new operating system.                   |
// |                                                |
// |        If you keep StartUp! without            |
// |        registering, you're stealing from me.   |
// |        And that isn't going to convince me to  |
// |        write more software for BeOS.  This nag |
// |        screen will probably drive you nuts     |
// |        too.                                    |
// |                                                |
// |             +-----------+  +----------------+  |
// |             | Enter key |  | Register later |  |
// |             +-----------+  +----------------+  |
// |                                                |
// +------------------------------------------------+

// ----------------------------------------------------------------------
// Constructor/destructor

NagWindow::NagWindow( void )
	: MWindow( BRect( 100, 100, 101, 101 ),
			   static_cast<const char *>( be_app->AppResources()->LoadResource( B_STRING_TYPE, "nag:title", &_size ) ),
			   B_TITLED_WINDOW_LOOK, B_FLOATING_ALL_WINDOW_FEEL,
			   B_NOT_RESIZABLE | B_NOT_ZOOMABLE |
//			   B_WILL_ACCEPT_FIRST_CLICK | B_ASYNCHRONOUS_CONTROLS ),
			   B_ASYNCHRONOUS_CONTROLS ),
	  _sfx( NULL ),
	  _nag( NULL ),
	  _later_button( NULL )
{
	App *app = static_cast<App *>( be_app );
	app->RegisterWindow( this );

	// Load strings from resources...
	BResources *rsrc = be_app->AppResources();

	const char *nag_title = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "nag:title", &_size ) );
	const char *nag_text = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "nag:text", &_size ) );
	const char *enter_key_label = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "nag:enter key", &_size ) );
	const char *later_label = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "nag:register later", &_size ) );

	// Views we'll want to mess with
	MStringView *title = NULL;
	MTextView *text = NULL;

	// View hierarchy:
	//
	// MWindow
	// + MBorder
	//   + VGroup
	//     + HGroup
	//       + app icon view
	//       + VGroup
	//         + Space
	//         + MStringView - be_bold_font, size * 1.2
	//     + Space
	//     + HGroup
	//       + Space
	//       + MTextView - be_plain_font
	//     + Space
	//     + HGroup
	//       + Space
	//       + MButton
	//       + Space
	//       + MButton

	// We must babysit MTextView, because BTextView sucks.  Calculate the
	// height of a line, and the maximum length of the text we have; we'll
	// use max_width / 3.0 for the view width (it looks good to me), and
	// 12.0 * one_line (that is, 4 * ( # of lines ) + 2) as the height.  It's
	// not a perfect solution, there will probably be an extra line or two of
	// whitespace after the text, but that's not as ugly as clipping...
	font_height fh;
	be_plain_font->GetHeight( &fh );
	float one_line = fh.ascent + fh.descent + fh.leading;

	float max_width = 0;
	{
		BString s( "" );
		char *ptr = const_cast<char *>( nag_text );
		while( *ptr != '\0' ) {
			s.Append( *ptr, 1 );
			if( *ptr == '\n' || *ptr == '\r' ) {
				const char *str[1];
				str[0] = s.String();

				escapement_delta delta[1];
				
				BRect bounds[1];

				be_plain_font->GetBoundingBoxesForStrings( str, 1, B_SCREEN_METRIC, delta, bounds );
				if( bounds[0].Width() > max_width ) max_width = bounds[0].Width();
				s.SetTo( "" );
			}
			ptr++;
		}
	}

	MBorder *view = new MBorder(
						M_RAISED_BORDER, 16, "",
						new VGroup(
							new HGroup(
								new MBViewWrapper( new AppIconView ),
								new Space( minimax( 8, 0, 8, 65536 ) ),

								new VGroup(
									new Space,
									( title = new MStringView( nag_title ) ),
									NULL
								),
								NULL
							),
							
							new Space( minimax( 0, 16, 65536, 16 ) ),
							
							new HGroup(
								new Space( minimax( 32, 0, 32, 65536 ) ),
								new Space( minimax( 8, 0, 8, 65536 ) ),
								// This is stupid.  BTextView isn't smart
								// enough to adjust its own frame rectangle or
								// its own text rectangle.
								( text = new MTextView( minimax( max_width / 3.0, one_line * 14, 65536, 65536 ) ) ),
								NULL
							),
							
							new Space( minimax( 0, 16, 65536, 16 ) ),
							
							new HGroup(
								new Space( minimax( 32, 0, 65536, 65536 ) ),
								new MButton( enter_key_label, new BMessage( NAG_ENTER_KEY ) ),
								new Space( minimax( 8, 0, 8, 65536 ) ),
								( _later_button = new MButton( later_label, new BMessage( B_QUIT_REQUESTED ) ) ),
								NULL
							),
							
							NULL
						)
					);

	AddChild( view );

	// Adjust the views
	BFont title_font( be_bold_font );
	title_font.SetSize( title_font.Size() * 1.2 );
	title->SetFont( &title_font );

	text->MakeEditable( false );
	text->MakeResizable( false );
	text->MakeSelectable( false );
	text->SetAlignment( B_ALIGN_LEFT );
	text->SetAutoindent( false );
	text->SetStylable( false );
	text->SetWordWrap( true );
	text->SetDoesUndo( false );
	text->SetText( nag_text );

	_later_button->SetEnabled( false );

	// Center the window; this isn't the user's window, so we don't store its
	// preferred location.
	RecalcSize();
	BRect r = Frame();
	float x, y;
	{
		BRect screen = BScreen( this ).Frame();
		x = ( screen.Width() - r.Width() ) / 2.0;
		y = ( screen.Height() - r.Height() ) / 2.0;
	}
	MoveTo( x, y );

	// Open the window
	Show();

	// Tell us when to stop nagging
	StartupSettings *settings = app->GetSettings();
	BMessenger hermes( this );
	_nag = new BMessageRunner( hermes, 
							   new BMessage( NAG_STOP ), 
							   ( settings->CheckRuns() * 2 + 5 ) * 1000 * 1000, 
							   1 );
}

NagWindow::~NagWindow( void )
{
	if( _sfx ) delete _sfx;
	if( _nag ) delete _nag;		// do you delete an expired BMEssageRunner?

	App *app = static_cast<App *>( be_app );
	app->DeregisterWindow( this );
}
	
// ----------------------------------------------------------------------
// BWindow methods

void NagWindow::MessageReceived( BMessage *msg )
{
	switch( msg->what ) {
	case NAG_ENTER_KEY:
		// the user wants to enter their key
		be_app->PostMessage( msg );
		Close();
		break;

	case NAG_STOP:
		// the nag time has expired
		_later_button->SetEnabled( true );
		{
			App *app = static_cast<App *>( be_app );
			app->SetIsNagging( false );
		}
		break;

	default:
		MWindow::MessageReceived( msg );
		break;
	}
}

bool NagWindow::QuitRequested( void )
{
	App *app = static_cast<App *>( be_app );
	return ( _later_button->IsEnabled() || app->CheckKey() );
}
