// ----------------------------------------------------------------------
// StartUp! - Registration window
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// Display and manage the Register window.
// ----------------------------------------------------------------------

#include "App.h"
#include "AppIconView.h"
#include "NagWindow.h"
#include "RegistrationWindow.h"
#include "StartupSettings.h"

#include <app/Message.h>
#include <interface/Font.h>
#include <storage/Resources.h>
#include <support/Autolock.h>
#include <ctype.h>
#include <stdio.h>

#include "HGroup.h"
#include "MBViewWrapper.h"
#include "MBorder.h"
#include "MButton.h"
#include "MStringView.h"
#include "MTextControl.h"
#include "Space.h"
#include "VGroup.h"

// +-------------------------+
// | [ ]  Register StartUp!  |
// +-------------------------+---------------------+
// |                                               |
// |  +--+                                         |
// |  |1 |  Thank you for registering StartUp!     |
// |  +--+                                         |
// |                                               |
// |  Please enter your registration information:  |
// |                                               |
// |           Name: ____________________________  |
// |        Company: ____________________________  |
// |         Copies: ___________                   |
// |                                               |
// | Registration #: 00-0000-0000-00               |
// |                                               |
// |                                 +----------+  |
// |                                 | Register |  |
// |                                 +----------+  |
// |                                               |
// +-----------------------------------------------+

// ----------------------------------------------------------------------
// Locals

static const char *SETTING_POSITION = "reg:position";

// ----------------------------------------------------------------------
// Constructor/destructor

RegistrationWindow::RegistrationWindow( void )
	: MWindow( BRect( 100, 100, 101, 101 ),
			   static_cast<const char *>( be_app->AppResources()->LoadResource( B_STRING_TYPE, "reg:title", &_size ) ),
			   B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
			   B_NOT_RESIZABLE | B_NOT_ZOOMABLE |
//			   B_WILL_ACCEPT_FIRST_CLICK | B_ASYNCHRONOUS_CONTROLS ),
			   B_ASYNCHRONOUS_CONTROLS ),
	  _name_field( NULL ),
	  _company_field( NULL ),
	  _copies_field( NULL ),
	  _key_field( NULL ),
	  _name( "" ),
	  _company( "" ),
	  _copies( "" ),
	  _key( "" )
{
	App *app = static_cast<App *>( be_app );
	app->RegisterWindow( this );
	
	// Load strings from resources...
	BResources *rsrc = be_app->AppResources();
	
	const char *reg_thanks = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:thank you", &_size ) );
	const char *reg_please = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:please enter", &_size ) );
	char *reg_name_label = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:name label", &_size ) ) );
	char *reg_company_label = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:company label", &_size ) ) );
	char *reg_copies_label = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:copies label", &_size ) ) );
	char *reg_key_label = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:key label", &_size ) ) );
	const char *reg_button_label = static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:button", &_size ) );
	char *reg_name_blank = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:blank name", &_size ) ) );
	char *reg_company_blank = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:blank company", &_size ) ) );
	char *reg_copies_blank = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:blank copies", &_size ) ) );
	char *reg_key_blank = const_cast<char *>( static_cast<const char *>( rsrc->LoadResource( B_STRING_TYPE, "reg:blank key", &_size ) ) );

	_name.SetTo( reg_name_blank );
	_company.SetTo( reg_company_blank );
	_copies.SetTo( reg_copies_blank );
	_key.SetTo( reg_key_blank );

	// Views we'll want to mess with
	MStringView *title = NULL;

	// View hierarchy:
	//
	// MWindow
	// + MBorder
	//   + VGroup
	//     + HGroup
	//       + app icon view
	//       + VGroup
	//         + Space
	//         + MStringView - be_bold_font, size * 1.2
	//     + Space
	//     + MStringView
	//     + Space
	//     + HGroup
	//       + VGroup
	//         + MTextControl
	//         + MTextControl
	//         + MTextControl
	//         + Space
	//         + MTextControl
	//     + Space
	//     + HGroup
	//       + Space
	//       + MButton

	MBorder *view = new MBorder(
						M_RAISED_BORDER, 16, "",
						new VGroup(
							new HGroup(
								new MBViewWrapper( new AppIconView ),
								new Space( minimax( 8, 0, 8, 65536 ) ),

								new VGroup(
									new Space,
									( title = new MStringView( reg_thanks ) ),
									NULL
								),
								NULL
							),
							
							new Space( minimax( 0, 16, 65536, 16 ) ),
							
							new MStringView( reg_please ),
							
							new Space( minimax( 0, 16, 65536, 16 ) ),

							( _name_field = new MTextControl( reg_name_label, reg_name_blank, new BMessage( REG_EDITED_NAME ) ) ),
							( _company_field = new MTextControl( reg_company_label, reg_company_blank, new BMessage( REG_EDITED_COMPANY ) ) ),
							( _copies_field = new MTextControl( reg_copies_label, reg_copies_blank, new BMessage( REG_EDITED_COPIES ) ) ),
							new Space( minimax( 0, 16, 65536, 16 ) ),
							( _key_field = new MTextControl( reg_key_label, reg_key_blank, new BMessage( REG_EDITED_KEY ) ) ),
							
							new Space( minimax( 0, 16, 65536, 16 ) ),
							
							new HGroup(
								new Space( minimax( 0, 0, 65536, 65536, 3 ) ),
								new MButton( reg_button_label, new BMessage( B_QUIT_REQUESTED ) ),
								NULL
							),

							NULL
						)
					);

	AddChild( view );

	BFont title_font( be_bold_font );
	title_font.SetSize( title_font.Size() * 1.2 );
	title->SetFont( &title_font );

	// Modification messages for the fields.
	_name_field->SetModificationMessage( new BMessage( REG_EDITED_NAME ) );
	_company_field->SetModificationMessage( new BMessage( REG_EDITED_COMPANY ) );
	_copies_field->SetModificationMessage( new BMessage( REG_EDITED_COPIES ) );
	_key_field->SetModificationMessage( new BMessage( REG_EDITED_KEY ) );

	// Align the fields properly.
	_name_field->SetAlignment( B_ALIGN_RIGHT, B_ALIGN_LEFT );
	_company_field->SetAlignment( B_ALIGN_RIGHT, B_ALIGN_LEFT );
	_copies_field->SetAlignment( B_ALIGN_RIGHT, B_ALIGN_LEFT );
	_key_field->SetAlignment( B_ALIGN_RIGHT, B_ALIGN_LEFT );

	// Make the fields not accept styling info.
	_name_field->TextView()->SetStylable( false );
	_company_field->TextView()->SetStylable( false );
	_copies_field->TextView()->SetStylable( false );
	_key_field->TextView()->SetStylable( false );

	DivideSame( _name_field, _company_field, _copies_field, _key_field, NULL );

	// This doesn't do anything.
	BTextView *key_text = _key_field->TextView();
	key_text->SetFont( be_fixed_font );

	_name_field->MakeFocus( true );

	// Restore position from settings
	BPoint pos;
	StartupSettings *settings = app->GetSettings();
	if( settings->FindPoint( SETTING_POSITION, &pos ) == B_OK ) {
		if( pos.x != 0.0 && pos.y != 0.0 ) MoveTo( pos );
	}

	// Display the window in all its glory!
	Show();
}

RegistrationWindow::~RegistrationWindow( void )
{
	App *app = static_cast<App *>( be_app );
	app->DeregisterWindow( this );
}

// ----------------------------------------------------------------------
// BWindow methods

void RegistrationWindow::MessageReceived( BMessage *msg )
{
	switch( msg->what ) {
	case REG_EDITED_NAME:
		_name.SetTo( _name_field->Text() );
		break;

	case REG_EDITED_COMPANY:
		_company.SetTo( _company_field->Text() );
		break;

	case REG_EDITED_COPIES:
		_copies.SetTo( _copies_field->Text() );
		break;

	case REG_EDITED_KEY:
		_key.SetTo( _key_field->Text() );
		break;
		
	default:
		MWindow::MessageReceived( msg );
		break;
	}
}

bool RegistrationWindow::QuitRequested( void )
{
	App *app = static_cast<App *>( be_app );

	// Strip extra whitespace from the front/end of the strings.
	StripWhitespace( _name );
	StripWhitespace( _company );
	StripWhitespace( _copies );
	StripWhitespace( _key );

	bool registered = app->TestKey( _name, _company, _copies, _key );
	if( !registered ) {
		// if we're not registered, we should display the nag window again
		be_app->PostMessage( NAG_SHOW_WINDOW );
	}

	StartupSettings *settings = app->GetSettings();
	settings->SavePoint( SETTING_POSITION, Frame().LeftTop() );

	return true;
}

void RegistrationWindow::StripWhitespace( BString& str )
{
	char *buff = str.LockBuffer( 0 );

	// count whitespace at the front
	int off_start = 0;
	while( isspace( buff[off_start] ) ) off_start++;

	// count whitespace at the end
	int off_end = strlen( buff ) - 1;
	while( isspace( buff[off_end] ) ) off_end--;

	BString ret( &(buff[off_start]), ( off_end - off_start + 1 ) );

	str.UnlockBuffer();

	str.SetTo( ret );
}
