// ----------------------------------------------------------------------
// StartupItem.h
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// An item in the StartUp! items folder; can tell you its arguments and
// entry_ref.
// ----------------------------------------------------------------------

#ifndef StartUp_StartupItem_H
#define StartUp_StartupItem_H

#include "CLVEasyItem.h"

#include <app/Message.h>
#include <storage/Mime.h>
#include <storage/Node.h>
#include <support/SupportDefs.h>

class BBitmap;
class BEntry;

enum {
	COL_ICON = 0,			// item icon
	COL_NAME,				// item name
	COL_ARGS,				// item arguments
	COL_DIRECTORY,			// item directory
//	COL_WORKSPACE,			// item workspace

	COL_COUNT				// # of columns
};

class StartupItem : public CLVEasyItem
{
public:
	StartupItem( BEntry *entry );
	virtual ~StartupItem( void );

	// StartupItem methods

	status_t InitCheck( void ) { return _status; }

	// Launch me!
	status_t Launch( void );

	// Info about this item
	BBitmap *GetIcon( icon_size which );
	const char *GetName( void );
	const char *GetArgs( void );
	bool CheckNode( node_ref &noder );

	// Settings we know about.
	int GetArgc( void );				// argc, if we have any command-line args
	char **GetArgv( void );				// argv, if we have any command-line args
	const char *GetDirectory( void );	// start-in-this-directory, if present

	bool SetArgs( const char *args );
	bool SetDirectory( const char *dir );
	bool SetName( const char *name );
	bool SetPrefs( BMessage &msg );		// get args, directory from message

private:
	StartupItem( const StartupItem& );		// don't copy me
	StartupItem& operator=( StartupItem& );	// don't assign me

	BEntry *_entry;				// A copy of our entry
	BMessage *_settings;		// StartUp! settings, including args if any
	BBitmap *_large_icon;		// my large icon
	BBitmap *_mini_icon;		// my mini icon
	const char *_name;			// file name
	const char *_args;			// presentable args
	const char *_directory;		// directory to start it in

	char **_argv;				// argv array, if we have any args

	status_t _status;			// Are we OK?

	node_ref _noder;			// my node_ref, for the Node Monitor

	bool LoadSettings( void );	// Load/unflatten our settings
	bool SaveSettings( void );	// Flatten/save the settings
};

#endif
