// ----------------------------------------------------------------------
// URLStringView.cpp
//
// Copyright © 2000 Arcane Dragon Software (arcanedragon@pobox.com)
//
// MStringView that's clickable, and launches its associated URL.
// ----------------------------------------------------------------------

#include "URLStringView.h"

#include <app/Message.h>
#include <app/Roster.h>
#include <support/Errors.h>
#include <malloc.h>
#include <string.h>

#include <stdio.h>

// ----------------------------------------------------------------------
// Constructor/destructor

URLStringView::URLStringView( const char *label, const char *url, 
				   alignment a, minimax size )
	: MStringView( label, a, size ),
	  _url( NULL )
{
	_url = const_cast<const char *>( strdup( url ) );
}

URLStringView::~URLStringView( void )
{
	if( _url ) free( const_cast<char *>( _url ) );
}


// ----------------------------------------------------------------------
// URL hackery

const char *URLStringView::GetURL( void )
{
	return _url;
}

void URLStringView::SetURL( const char *url )
{
	if( _url ) free( const_cast<char *>( _url ) );
	_url = const_cast<const char *>( strdup( url ) );
}

status_t URLStringView::VisitURL( void )
{
	int offset = strchr( _url, ':' ) - _url;
	const char *type = NULL;
	if( !strncmp( _url, "mailto", offset ) ) {
		// Or should it be message/rfc822?  Sure wish people would stop
		// putting random MIME types on my system...
		type = "text/x-email";
	} else if( !strncmp( _url, "http", offset ) ) {
//		type = "x-vnd.Be.URL.http";
		type = "text/html";
	} else if( !strncmp( _url, "https", offset ) ) {
//		type = "x-vnd.Be.URL.https";
		type = "text/html";
	} else if( !strncmp( _url, "ftp", offset ) ) {
		type = "x-vnd.Be.URL.ftp";
	} else if( !strncmp( _url, "file", offset ) ) {
		type = "x-vnd.Be.URL.file";
	} else if( !strncmp( _url, "rtsp", offset ) ) {
		type = "x-vnd.Be.URL.rtsp";
	} else if( !strncmp( _url, "telnet", offset ) ) {
		type = "x-vnd.Be.URL.telnet";
	} else {
		type = "x-vnd.Be.URL.http";
	}

	status_t retval = be_roster->Launch( type, 1, const_cast<char **>( &_url ) );
	printf( "retval for URL launch was: %s\n", strerror( retval ) );

	if( retval == B_ALREADY_RUNNING ) retval = B_OK;
	return retval;
}

// ----------------------------------------------------------------------
// BView methods
void URLStringView::AttachedToWindow( void )
{
	// Just in case.
	MStringView::AttachedToWindow();

	rgb_color blue = { 0, 0, 0xff, 0 };
	SetHighColor( blue );
}

void URLStringView::MouseDown( BPoint point )
{
	status_t retval = VisitURL();
	if( retval != B_OK ) {
		// it didn't work...
	}
}
